/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file C0RectSet.hpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library,
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details.

#ifndef _CAPD_DYNSET_C0RECTSET_HPP_
#define _CAPD_DYNSET_C0RECTSET_HPP_

#include "capd/vectalg/iobject.hpp"
#include "capd/dynset/C0RectSet.h"
#include "capd/matrixAlgorithms/floatMatrixAlgorithms.hpp"


namespace capd{
namespace dynset{

template<typename MatrixType>
void C0RectSet<MatrixType>::move(DynSysType & dynsys) {
  move(dynsys,*this);
}

template<typename MatrixType>
void C0RectSet<MatrixType>::move(DynSysType & dynsys, C0RectSet<MatrixType>& result) const {

  VectorType xx = m_x + m_B*m_r;

  result.m_x = dynsys.Phi(m_x) + dynsys.Remainder(xx);
  MatrixType D = dynsys.JacPhi(xx)*m_B;
// xx is unnecessary, we can use it again
  split(result.m_x,xx);
  mid(D,result.m_B);
  capd::matrixAlgorithms::orthonormalize(result.m_B,m_r);
  MatrixType Qtr = Transpose(result.m_B);
  result.m_r = (Qtr*D)*m_r + Qtr*xx;
}

template<typename MatrixType>
std::string C0RectSet<MatrixType>::show(void) const
{
  //std::ostringstream descriptor;
  //descriptor << "C0RectSet:\n" << toString();
  //return descriptor.str();
  return "C0RectSet:\n" + BaseSet::toString();
}


}} // namespace capd::dynset

#endif // _CAPD_DYNSET_C0RECTSET_HPP_

/// @}


















