/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file C2Taylor.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

/* Author: Daniel Wilczak 2003-2005 */

#ifndef _CAPD_DYNSYS_C2TAYLOR_H_ 
#define _CAPD_DYNSYS_C2TAYLOR_H_ 

#include <string>
#include <vector>
#include "capd/vectalg/Norm.h"
#include "capd/dynsys/Taylor.h"
#include "capd/dynsys/C2DynSys.h"
#include "capd/dynsys/BasicC2Taylor.h"

namespace capd{
namespace dynsys{

template<typename MapT>
class C2Taylor : public Taylor<MapT>, public BasicC2Taylor<MapT>, public C2DynSys<typename MapT::MatrixType>{
public:
  typedef MapT MapType;
  typedef typename MapT::FunctionType FunctionType;
  typedef typename MapType::MatrixType MatrixType;
  typedef typename MatrixType::RowVectorType VectorType;
  typedef typename MatrixType::ScalarType ScalarType;
  typedef capd::vectalg::Norm<VectorType,MatrixType> NormType;
  typedef typename MapType::C2CoeffType C2CoeffType;

  C2Taylor(MapType& vectorField, int order, const ScalarType& step);

  void c2Phi(const VectorType& x, MatrixType& jac, C2CoeffType& hessian) const;
  void c2Enclosure(
         const VectorType& W1,
         const MatrixType& W3,
         const NormType& the_norm,
         C2CoeffType& result
  ) const;
  void c2Remainder(
         const VectorType& Enclosure,
         const MatrixType& jacEnclosure,
         const C2CoeffType& hessianEnclosure,
         VectorType& Remainder,
         MatrixType& jacRemainder,
         C2CoeffType& hessianRemainder // here the result is stored
  ) const;

  using BasicC2Taylor<MapType>::setOrder;

  using Taylor<MapType>::Phi;
  using Taylor<MapType>::JacPhi;
  using Taylor<MapType>::Remainder;
  using Taylor<MapType>::JacRemainder;
  using Taylor<MapType>::enclosure;
  using Taylor<MapType>::jacEnclosure;

  using Taylor<MapType>::getField;
  using Taylor<MapType>::getOrder;
  using Taylor<MapType>::getStep;
  using Taylor<MapType>::setStep;
  using Taylor<MapType>::getCoeff;
  using Taylor<MapType>::getMatrixCoeff;

  void differentiateTime() const
  {
    Taylor<MapType>::differentiateTime();
  }
  void setCurrentTime(const ScalarType& a_time) const
  {
    Taylor<MapType>::setCurrentTime(a_time);
  }

  const ScalarType& getCurrentTime(const ScalarType& a_time) const
  {
    return Taylor<MapType>::getCurrentTime();
  }

protected:
  void operator=(const C2Taylor& ) {}
  C2Taylor(const C2Taylor& t) : BasicTaylor<MapT>(t), Taylor<MapT>(t), BasicC2Taylor<MapT>(t) {}

  using Taylor<MapType>::computeMatrixCoeff;
  using Taylor<MapType>::m_order;
  using Taylor<MapType>::m_dimension;
  using Taylor<MapType>::m_step;
  using Taylor<MapType>::m_vField;
  using Taylor<MapType>::m_coeff;
  using Taylor<MapType>::m_matrixCoeff;
  using Taylor<MapType>::m_F;

  using BasicC2Taylor<MapType>::m_c2MatrixCoeff;
  using BasicC2Taylor<MapType>::computeC2Coeff;
}; // the end of class C2Taylor

}} // the end of the namespace capd::dynsys

#endif // _CAPD_DYNSYS_C2TAYLOR_H_ 

/// @}
