#include "saveToFile.h"
using std::ios;
using namespace std;

void saveInitialConditions(char * name, int iter, intervalVector &x0, intervalVector &X, intervalVector &x0_e, intervalVector &X_e)
{
 ofstream pl(name,ios::app);
 pl<<setprecision(12)<<"\n==================================================\n"
      <<" INITIAL CONDITIONS for iteration " << iter <<" \n==================================================\n"
      << "\n In reduced space \n\n x0 = " << x0
      <<"\n\n X = " <<X <<"\n\n diam X "<<(X.size())
      <<"\n\n In full space\n\n  E(x0) = " <<x0_e
      <<"\n\n E(X) = " <<X_e<<"\n\n diam E(X) "<<(X_e.size())<<endl;
    pl.close();

}

void saveInitialConditions(char * name, int iter, intervalVector &x0, intervalVector &X)
{
 ofstream pl(name,ios::app);
 pl<<setprecision(12)<<"\n==================================================\n"
      <<" INITIAL CONDITIONS for iteration " << iter <<" \n==================================================\n"
      << "\n\n x0 = " << x0
      <<"\n\n X = " <<X <<"\n\n diam X "<<(X.size());
     pl.close();

}
// For intervalMatrix A returns
//  max - absolute value of the biggest entry
//   average - average  of  entries value
//   maxDiam - maximal diameter of its coordinates
//  averageDiam - average diameter

void  matrixTest(intervalMatrix &A, double & max, double &average, double &maxDiam, double &averageDiam)
{
 max=abs(A[0][0]).right_b();
  average =0;
  averageDiam =0;
  maxDiam = 0;

   for(int i = 0; i< A.get_dim1(); i++)
   {
       vector diam = A[i].diam();
       for(int j = 0; j< A.get_dim2(); j++)
       {
          if(max<abs(A[i][j]).right_b())
            max = abs(A[i][j]).right_b();
          average+= abs(A[i][j]).right_b();

          if(maxDiam < diam[j])
            maxDiam = diam[j];
          averageDiam+= diam[j];

       }
     }
     averageDiam /= (A.get_dim1()*A.get_dim2());
      average /= (A.get_dim1()*A.get_dim2());

}


// It saves to file fileName information about Matrix
//  such as maximal and average value and diameter of its cooeficient
//
void saveMatrixInfo(char* fileName, char* matrixName, intervalMatrix &A)
{

 double max, average, maxDiam, averageDiam;
 matrixTest(A,  max, average, maxDiam, averageDiam);
 ofstream pl(fileName,ios::app);
 pl <<"\n matrix " << setprecision(16) << matrixName
     <<"\n" << A
       << "\n max value        =" <<max
       << "\n average value    = " << average
       << "\n max diameter     = " << maxDiam
       << "\n average diameter = " << averageDiam
       << " \n diameters of first row of " << matrixName << "\n  "<< A[0].diam()<<endl;
 pl.close();
}

void saveInfo(char* fileName, char* vectorName, intervalVector &v, int prec )
{

 ofstream pl(fileName,ios::app);
 if(prec)
      pl <<setprecision(prec);
 pl <<"\n vector " << vectorName
     <<"\n"  << v
       << "\n max diameter     = " << v.size()
       << " \n diam :" << v.diam()<<endl;
 pl.close();
}

void saveInfo(char* fileName, char* intervalName, interval &v)
{

 ofstream pl(fileName,ios::app);
 pl  << intervalName
     <<"\n" << v
       << "\n diameter     = " << diameter(v).right_b();
 pl.close();
}

void saveInfo(char*fileName, char* text, double time)
{
    ofstream pl(fileName,ios::app);
 saveInfo(pl, text, time);
 pl.close();
}

void saveParameters(char*fileName, char* text,    double step, int order, double X_step, int X_order)
{
 ofstream pl(fileName,ios::app);
 pl <<endl <<  text <<endl << "for point: step = " <<step <<"    order = " << order <<endl;
 pl << "for set: step = " <<X_step <<"    order = " << X_order<<endl;
 pl.close();
}
