
#ifndef _TYPEDEFS_H_
#define _TYPEDEFS_H_

#define MP 2
// MP defines type of interval arithmetics used:
//  1 - multiprecision intervals class 
//               capd::multiPrec::Interval class
//  2 - template multiprecision class
//               capd::intervals::Interval<MpfrClass> 
//  otherwise  - intervals with double ends 
//               capd::intervals::Interval<double, doubleRounding>


#if(MP == 1)
#include "capd/multiPrec/MpInterval.h"
#elif (MP == 2)
#include "capd/multiPrec/MpfrClass.h"
#include "capd/interval/Interval.hpp"
#include "capd/interval/MpInterval.h"
#else
#include "capd/interval/DoubleInterval.h"
#endif

#include "capd/vectalg/Vector.hpp"
#include "capd/vectalg/Matrix.hpp"
#include "capd/vectalg/Norm.hpp"

#include "capd/map/Function.hpp"
#include "capd/map/Map.hpp"
#include "capd/map/CnMap.hpp"

#include "capd/dynsys/Taylor.hpp"
#include "capd/dynsys/BasicTaylor.hpp"

#include "capd/dynset/Rect2Set.hpp"
#include "capd/dynset/RectSet.hpp"
#include "capd/dynset/C2Rect2.hpp"
#include "capd/dynset/C1Rect2.hpp"
#include "capd/dynset/PpedSet.hpp"
#include "capd/dynset/Pped2Set.hpp"

#include "capd/poincare/TimeMap.hpp"
#include "capd/poincare/TimeMapTSC.hpp"


#define DIMENSION 0
// Dimension is not fixed (can be dynamically changed).

#if (MP == 1)
typedef capd::multiPrec::MpfrClass ScalarType;
typedef capd::multiPrec::Interval Interval;
#elif (MP == 2)
typedef capd::multiPrec::MpfrClass ScalarType;
typedef capd::intervals::Interval<ScalarType> Interval;

#else
typedef interval Interval;
typedef double ScalarType;
#endif

typedef capd::vectalg::Vector<ScalarType, DIMENSION > FloatVector;
typedef capd::vectalg::Vector<Interval, DIMENSION > IntervalVector;
typedef capd::vectalg::Matrix<ScalarType, DIMENSION, DIMENSION > FloatMatrix;
typedef capd::vectalg::Matrix<Interval, DIMENSION, DIMENSION > IntervalMatrix;

typedef capd::map::Function<IntervalVector> FloatFunction;
typedef capd::map::Function<FloatVector> Function;

typedef capd::map::CnMap<FloatMatrix> FloatMap;
typedef capd::map::CnMap<IntervalMatrix> Map;

typedef capd::dynsys::Taylor<Map> Taylor;
typedef capd::dynsys::BasicTaylor<FloatMap> FloatTaylor;

typedef capd::poincare::TimeMap<Taylor> TimeMap;
typedef capd::poincare::TimeMapTSC<Taylor> TimeMapTSC;
typedef capd::poincare::TimeMap<FloatTaylor> FloatTimeMap;

typedef capd::vectalg::EuclLNorm<IntervalVector,IntervalMatrix> LNorm;
typedef capd::dynset::C0Set<IntervalMatrix> C0set;
//typedef capd::dynset::RectSet<intervalMatrix> RectSet;
//typedef capd::dynset::Rect2Set<intervalMatrix> Rect2Set;
//typedef capd::dynset::Intv2Set<intervalMatrix> Intv2Set;
//typedef capd::dynset::IntervalSet<intervalMatrix> IntervalSet;
//typedef capd::dynset::PpedSet<intervalMatrix> PpedSet;
//typedef capd::dynset::C2Rect2<intervalMatrix> C2rect2;
typedef capd::dynset::C1Rect2<IntervalMatrix> C1rect2;

#endif //_TYPEDEFS_H_
