/******************************************************************************/
//  Functions used to write report to the TeX file
/******************************************************************************/

void tex_interval(ostream &pl, interval v, char *name, int prec = 10)
{
    pl <<" "<< name <<" &  \\mbox{" <<setprecision(prec) << v << "} \\\\ \n";
}

void tex_vector(ostream &pl, vector v, char *name, int prec = 10)
{
    pl <<" "<< name <<" & $ \n   \\left(\\begin{array}{c} \n";
   for( int i = 0; i <v.getDim(); i++)
     pl << "\\mbox{" <<setprecision(prec) << v[i] << "} \\\\ \n";
   pl<< "\\end{array} \\right)$ \\\\ \n ";

}

void tex_vector2(ostream &pl, vector v, char *name, int prec = 10)
{
   pl <<" "<< name <<" & $ \n   \\left(";
   pl << "\\mbox{" <<setprecision(prec) << v[0] << "}";
   for( int i = 1; i <v.getDim(); i++)
     pl << ", \\mbox{ " <<setprecision(prec) << v[i] << "}";
   pl<< "\\right)$ \\\\ \n ";

}

void tex_ivector(ostream &pl, intervalVector v, char *name, int prec = 10)
{
    pl <<" "<< name <<" & $ \n   \\left(\\begin{array}{c} \n";
   for( int i = 0; i <v.getDim(); i++)
     pl <<"\\mbox{"<< setprecision(prec) << v[i] << "} \\\\ \n";
   pl<< "\\end{array} \\right)$ \\\\ \n ";

}

void tex_imatrix(ostream &pl, intervalMatrix v, char *name, int prec = 10)
{
    pl <<" "<< name <<" & $ \n   \\left(\\begin{array}{";
         for( int j=0; j < v.nColumns(); j++) pl <<"c";
    pl<<"} \n";
   for( int i = 0; i <v.nRows(); i++)
   {
     pl <<"\\mbox{"<< setprecision(prec) << v[i][0] << "} ";
     for( int j=1; j < v.nColumns(); j++)
       pl <<"& \\mbox{"<< setprecision(prec) << v[i][j] << "} ";
     pl<<" \\\\ \n";
   }
   pl<< "\\end{array} \\right)$ \\\\ \n ";

}

std::string midRadiusForm(interval v, int prec = 2)
{

  std::ostringstream str;
  double m = (v.right_b() + v.left_b())/2,
         r = (v.right_b() - v.left_b())/2;

  str << setiosflags(ios::fixed) << setprecision(prec) << m <<" \\pm " <<setiosflags(ios::scientific)<< setprecision(prec)<< r ;
  return str.str();
}

void tex_imatrix_s(ostream &pl, intervalMatrix v, char *name, int prec = 10)
{
    pl <<" "<< name <<" & $ \n   \\left(\\begin{array}{";
         for( int j=0; j < v.nColumns(); j++) pl <<"c";
    pl<<"} \n";
   for( int i = 0; i <v.nRows(); i++)
   {
     pl <<"\\mbox{\\scriptsize{"<< setprecision(prec) << midRadiusForm(v[i][0]) << "}} ";
     for( int j=1; j < v.nColumns(); j++)
       pl <<"& \\mbox{\\scriptsize{"<< setprecision(prec) << midRadiusForm(v[i][j]) << "}} ";
     pl<<" \\\\ \n";
   }
   pl<< "\\end{array} \\right)$ \\\\ \n ";

}

//void writeToTEX(char *name, const intervalVector &x0, const intervalVector &X,
//                const intervalVector &F_x0, const intervalMatrix &dF_X, const intervalVector &K,
 //               double step, int order, double X_step, int X_order, double czas, double czas1, int iter)
void writeToTEX(char *name, intervalVector &x0, intervalVector &X,
                intervalVector &F_x0, intervalMatrix &dF_X, intervalVector &K,
                double step, int order, double X_step, int X_order, double czas, double czas1, int iter)

{
    ofstream file(name, ios::app);
    file<<setprecision(16)
      << "\\begin{table} \\begin{center} \n\\begin{tabular}{|c|l|}\\hline \n"
      <<"\\multicolumn{2}{|c|}{Wartoci pocztkowe}\\\\ \\hline \n& \\\\";
    tex_vector2(file, x0.getVector() ,"$x_0$");
    file << "&\\\\";
    tex_ivector(file, X, "$X$");
    file << "&\\\\";
    tex_vector2(file, (X.diam()), "diam $[X]$", 4);
    file << "&\\\\\\hline"
         <<"\\multicolumn{2}{|c|}{Parametry oblicze}\\\\ \\hline \n& \\\\"
         << "\n dla punktu &  krok czasowy : " << step
         << ",  rzd metody : " << order
         << " \\\\\n dla zbioru & krok czasowy : " << X_step
         << ",  rzd metody : " << X_order
         <<"\\\\ \\hline \n \\end{tabular} \n \\caption{Wartoci pocztkowe}\n\\end{center}\n\\end{table}"
         <<" \n\n  \\begin{table} \n\\begin{tabular}{|c|l|}\\hline \n"
         << " \\multicolumn{2}{|c|}{Wyniki oblicze}  \\\\\\hline \n"
         << "&\\\\";
    tex_ivector(file, F_x0 ," $\\hat{F}(\\bar{z})$", 12);
    file << "&\\\\";
    tex_vector2(file, F_x0.diam() ," $\\diam \\hat{F}(\\bar{z})$", 4);
    file << "&\\\\";
    tex_imatrix_s(file, dF_X ," $\\frac{\\partial \\hat{F}}{\\partial z}(Z)$", 3);
    file << "&\\\\";
    tex_ivector(file, K, "$K(\\bar{x},X,\\Phi)$",16);
    file << "&\\\\";
    tex_vector2(file, (K.diam()), "$\\diam K(\\bar{x},X,\\Phi)$",4);
    file <<"&\\\\" << "\n Czas oblicze & dla punktu : " <<setprecision(3) <<czas1
         << " sek. \\\\ \n &  dla zbioru " <<czas <<" sek. \\\\&"
         << "\\\\ \\hline \n \\multicolumn{2}{|c|} { Liczba iteracji Przedziaowej Metody Krawczyka : "
         << iter <<"} \\\\" <<"\\hline \n ";
    file.close();
}


